﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Text;

namespace TruongPham.WebControls
{
    public enum PanelState
    {
        Expand,
        Collapse
    }

    [DefaultProperty("Text")]
    [ToolboxData("<{0}:PDTPanel runat=server></{0}:PDTPanel>")]
    [ToolboxBitmap(typeof(PDTPanel), "WebControls.PDTPanel.bmp")]
    [Description("Draws a rounded, configurable panel with grouping text.")]
    [ParseChildren(false)]
    [PersistChildren(true)]
    public class PDTPanel : Control 
    {
        #region [ Attributes ]
        private string _controlGUID = System.Guid.NewGuid().ToString("P");
        private ArrayList _items = new ArrayList(); //Retain all the child/nested controls inside panel
        private int _maxWidth = -1;
        #endregion

        #region [ Properties ]
        private string _contentRegionid = string.Empty;
        private string ContentRegionID
        {
            get
            {
                if (_contentRegionid == string.Empty) _contentRegionid = "contentRegion_" + _controlGUID;
                return _contentRegionid;
            }
        }

        private string _groupingTitle = "";
        public string GroupingTitle
        {
            get
            {
                return _groupingTitle;
            }
            set
            {
                _groupingTitle = value;
            }
        }

        private string _groupingIconUrl = "";
        [UrlProperty()]
        public string GroupingIconUrl
        {
            get
            {
                return _groupingIconUrl;
            }
            set
            {
                _groupingIconUrl = value;
            }
        }

        private bool _shadow = false;
        public bool Shadow
        {
            get
            {
                return _shadow;
            }
            set
            {
                _shadow = value;
            }
        }

        private string _width = "";
        public string Width
        {
            get
            {
                return _width;
            }
            set
            {
                _width = value;
            }
        }

        private string _adjustedWidth = "";
        public string AdjustedWidth
        {
            get
            {
                return _adjustedWidth;
            }
            set
            {
                _adjustedWidth = value;
            }
        }

        private bool _allowExpandCollapse = false;
        public bool AllowExpandCollapse
        {
            get
            {
                return _allowExpandCollapse;
            }
            set
            {
                _allowExpandCollapse = value;
            }
        }

        private PanelState _initialState = PanelState.Expand;
        public PanelState InitialState
        {
            get
            {
                return _initialState;
            }
            set
            {
                _initialState = value;
            }
        }


        private string _expandIconUrl = "";
        [UrlProperty()]
        public string ExpandIconUrl
        {
            get
            {
                return _expandIconUrl;
            }
            set
            {
                _expandIconUrl = value;
            }
        }

        private string _collapseIconUrl = "";
        [UrlProperty()]
        public string CollapseIconUrl
        {
            get
            {
                return _collapseIconUrl;
            }
            set
            {
                _collapseIconUrl = value;
            }
        }

        private string _expandTooltip = "Click to expand";
        public string ExpandTooltip
        {
            get
            {
                return _expandTooltip;
            }
            set
            {
                _expandTooltip = value;
            }
        }

        private string _collapseTooltip = "Click to collapse";
        public string CollapseTooltip
        {
            get
            {
                return _collapseTooltip;
            }
            set
            {
                _collapseTooltip = value;
            }
        }

        private string _backgroundUrl = "";
        [UrlProperty()]
        public string BackgroundUrl
        {
            get
            {
                return _backgroundUrl;
            }
            set
            {
                _backgroundUrl = value;
            }
        }

        private string _defaultContainer;
        public string DefaultContainer
        {
            get
            {
                return _defaultContainer;
            }
            set
            {
                _defaultContainer = value;
            }
        }

        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("RoundedPanel")]
        [Localizable(true)]
        [Description("The inner text of the rounded panel.")]
        public string Text
        {
            get
            {
                String s = (String)ViewState["Text"];
                return ((s == null) ? String.Empty : s);
            }
            set
            {
                ViewState["Text"] = value;
            }
        }

        #endregion

        #region [ Overrides ]
        //Override AddParsedSubObject(): Notifies the server control that an element, either XML or HTML, was parsed, and adds the element to the server control's ControlCollection object.
        [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
        protected override void AddParsedSubObject(Object obj)
        {
            _items.Add(obj);
        }

        protected override void CreateChildControls()
        {
            System.Collections.IEnumerator myEnumerator = _items.GetEnumerator();
            while (myEnumerator.MoveNext())
            {
                Control ctr = (Control)myEnumerator.Current;
                this.Controls.Add(ctr);
            }
        }

        [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
        protected override void Render(HtmlTextWriter output)
        {
            StringBuilder sbOut = new StringBuilder();
            string imageWebResourceUrl;
            int colspan;

            sbOut.AppendLine("<table style='WIDTH:" + CalculatePanelWidth() + "; BORDER-RIGHT: medium none; BORDER-TOP: medium none; BORDER-LEFT: medium none; BORDER-BOTTOM: medium none; BORDER-COLLAPSE: collapse' cellSpacing=0 cellPadding=0>");

            sbOut.AppendLine("  <tbody>");
            sbOut.AppendLine("      <tr style='height:8px'>");
            sbOut.AppendLine("          <td></td>");

            string title = this.GroupingTitle.Replace(" ", "&nbsp;"); //Replace all the empty char with HTML character "&nbsp;"
            if(_groupingIconUrl != "")
                title = String.Format("{0}&nbsp;<img src='{1}' />", title, _groupingIconUrl);

            if (GroupingTitle.Trim() != "" || _groupingIconUrl != "")
                sbOut.AppendLine("          <td rowspan='2' colspan='1' style='vertical-align:middle;width:auto' nowrap>" + title + "</td>");

            if (AdjustedWidth == "")
                sbOut.AppendLine("          <td style='width:100%'></td>");
            else
            {
                string __adjustedWidth = AdjustedWidth;
                if (__adjustedWidth.Contains("%") == false && __adjustedWidth.Contains("px") == false)
                    __adjustedWidth = __adjustedWidth + "px";

                __adjustedWidth = _maxWidth.ToString() + "px";
                sbOut.AppendLine("          <td style='width:" + __adjustedWidth + "'></td>");
            }


            //===========================================
            //Display Expand/Collapse Image if enabled
            if (AllowExpandCollapse == true)
            {
                sbOut.AppendLine("          <td rowspan='2'>");

                
                string __ExpandCollapseIcon;
                string __activeTooltip;

                if (_initialState == PanelState.Expand)
                {
                    __ExpandCollapseIcon = (CollapseIconUrl == "") ? Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images.icon_collapse.gif") : CollapseIconUrl;
                    __activeTooltip = _collapseTooltip;
                }
                else
                {
                    __ExpandCollapseIcon = (ExpandIconUrl == "") ? Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images.icon_expand.gif") : ExpandIconUrl;
                    __activeTooltip = _expandTooltip;
                }

                sbOut.AppendLine("             <div style='float:right;'><img id='imgExpandCollapse_" + _controlGUID + "'  alt='" + __activeTooltip + "' src='" + __ExpandCollapseIcon + "' border=0 onclick=\"" + ExpandCollapseScript() + "\" /></div>");
                sbOut.AppendLine("          </td><td></td>");
            }
            else
                sbOut.AppendLine("          <td style='width:auto'></td>");
            //===========================================

            sbOut.AppendLine("      </tr>");

            sbOut.AppendLine("      <tr>");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: justify'>");
            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailLeftTop.gif");
            sbOut.AppendLine("              <IMG alt='' src='" + imageWebResourceUrl + "' border=0></td>");


            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailTop.gif");

            if (this.Page.Request.Browser.Browser.ToLower() == "applemac-safari" //Safari, Google Chrome
               ||
                (this.Page.Request.Browser.Browser.ToLower() == "ie" && this.Page.Request.Browser.Version == "8.0") //IE 8.0
               )
                sbOut.AppendLine("          <td style='WIDTH:" + _maxWidth + "px; BACKGROUND: url(" + imageWebResourceUrl + ") repeat-x left top; PADDING-LEFT: 2px; PADDING-TOP: 12px; TEXT-ALIGN: center; VERTICAL-ALIGN: middle'></td>");
            else
                sbOut.AppendLine("          <td style='BACKGROUND: url(" + imageWebResourceUrl + ") repeat-x left top; PADDING-LEFT: 2px; PADDING-TOP: 12px; TEXT-ALIGN: center; VERTICAL-ALIGN: middle'></td>");


            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailRightTop.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: left'>");
            sbOut.AppendLine("               <IMG alt='' src='" + imageWebResourceUrl + "' border=0></td>");

            sbOut.AppendLine("      </tr>");

            sbOut.AppendLine("      <tr>");
            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailLeft.gif");
            sbOut.AppendLine("          <td style='BACKGROUND: url(" + imageWebResourceUrl + ") repeat-y left top; VERTICAL-ALIGN: top; WIDTH: 24px; HEIGHT: 20px; TEXT-ALIGN: justify'></td>");

            //===========================================
            colspan = 2;

            //If Expand/Collapse is disabled, shrink column from 3 to 2
            if (AllowExpandCollapse == true)
                colspan = colspan + 1;

            if (GroupingTitle.Trim() == "" && _groupingIconUrl == "")
                colspan = colspan - 1;

            sbOut.AppendLine("          <td colspan='" + colspan + "'>");
            //===========================================

            output.Write(sbOut.ToString());

            // Render child controls if necessary, nest in a P tag
            if (HasControls())
            {
                output.Write("<div style='width:" + _maxWidth + "px;height:1px;'></div>");
                
                //if (BackgroundUrl != "")
                //    output.Write("<div id='{0}' style='background-image:url({1})'>", ContentRegionID, BackgroundUrl);
                //else
                //    output.Write("<div id='{0}'>", ContentRegionID);

                output.Write("<div id='{0}' style='", ContentRegionID);

                if (_initialState == PanelState.Collapse)
                    output.Write("display:none");
                else
                    output.Write("display:block");

                if (BackgroundUrl != "")
                    output.Write(";background-image:url({0})", BackgroundUrl);

                output.Write("'>");
                               
                this.RenderChildren(output);
                output.Write("</div>");
            }

            sbOut = new StringBuilder();
            sbOut.AppendLine("          </td>");

            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailRight.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; BACKGROUND: url(" + imageWebResourceUrl + ") left top; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: left'></td>");

            sbOut.AppendLine("      </tr>");

            sbOut.AppendLine("      <tr>");

            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailLeft.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; BACKGROUND: url(" + imageWebResourceUrl + ") repeat-y left top; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: justify'></td>");

            //===========================================
            colspan = 2;

            //If Expand/Collapse is disabled, shrink column from 3 to 2
            if (AllowExpandCollapse == true)
                colspan = colspan + 1;

            if (GroupingTitle.Trim() == "" && _groupingIconUrl == "")
                colspan = colspan - 1;

            sbOut.AppendLine("          <td style='VERTICAL-ALIGN: top; TEXT-ALIGN: left' colspan='" + colspan + "'></td>");
            //===========================================


            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailRight.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; BACKGROUND: url(" + imageWebResourceUrl + ") left top; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: left'></td>");

            sbOut.AppendLine("      </tr>");
            sbOut.AppendLine("      <tr>");

            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailLeftBottom.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: justify'><IMG alt='' src='" + imageWebResourceUrl + "' border=0>");
            sbOut.AppendLine("           </td>");

            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailBottom.gif");
            
            //===========================================
            colspan = 2;

            //If Expand/Collapse is disabled, shrink column from 3 to 2
            if (AllowExpandCollapse == true)
                colspan = colspan + 1;

            if (GroupingTitle.Trim() == "" && _groupingIconUrl == "")
                colspan = colspan - 1;

            sbOut.AppendLine("          <td  colspan='" + colspan + "' style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; BACKGROUND: url(" + imageWebResourceUrl + ") repeat-x left top; VERTICAL-ALIGN: middle; BORDER-LEFT: medium none; BORDER-BOTTOM: medium none; HEIGHT: 24px; TEXT-ALIGN: center'></td>");

            //===========================================

            imageWebResourceUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images" + ((Shadow == true) ? ".WithShadow" : "") + ".DetailRightBottom.gif");
            sbOut.AppendLine("          <td style='BORDER-RIGHT: medium none; BORDER-TOP: medium none; VERTICAL-ALIGN: top; BORDER-LEFT: medium none; WIDTH: 24px; BORDER-BOTTOM: medium none; TEXT-ALIGN: left'><IMG alt='' src='" + imageWebResourceUrl + "' border=0></td>");

            sbOut.AppendLine("      </tr>");
            sbOut.AppendLine("  </tbody>");
            sbOut.AppendLine("</table>");

            output.Write(sbOut.ToString());
        }
        #endregion

        #region [ Private ]

        private string CalculatePanelWidth()
        {
            string widthDisplay;

            foreach (Object obj in _items)
            {
                if (obj is WebControl)
                {
                    WebControl wc = (WebControl)obj;

                    if (wc.Style["width"] != null)
                    {
                        int width = int.Parse(wc.Style["width"].Replace("px", ""));

                        if (width > _maxWidth) _maxWidth = width;
                    }
                    else
                    {
                        if (wc.Width != null && wc.Width.ToString() != "")
                        {
                            int width;
                            int.TryParse(wc.Width.Value.ToString(), out width);

                            if (width > _maxWidth) _maxWidth = width;
                        }
                    }
                }

                if (obj is LiteralControl)
                {
                    LiteralControl c = (LiteralControl)obj;

                    ArrayList widths = ExtractWidthTags(c.Text);

                    foreach (int width in widths)
                    {
                        if (width > _maxWidth) _maxWidth = width;
                    }
                }
            }

            //---------------------------------------------------------
            Width = Width.Replace("px", "").Replace(" ", "");

            _maxWidth = _maxWidth + 10;

            if (Width == "auto")
            {
                if (_maxWidth > -1)
                    widthDisplay = _maxWidth + "px";
                else
                    widthDisplay = "100%";
            }
            else
            {
                int w;
                int.TryParse(Width, out w);

                if (w > 0)
                {
                    //if (_maxWidth > -1)
                    //    if (_maxWidth > w)
                    //        widthDisplay = _maxWidth + "px";
                    //    else
                    //        widthDisplay = w + "px";
                    //else
                    //    widthDisplay = w + "px";

                    widthDisplay = w + "px";
                }
                else
                    widthDisplay = "100%";
            }

            return widthDisplay;
        }


        private ArrayList ExtractWidthTags(string s)
        {
            s = s.Replace(" ", "");

            ArrayList widths = new ArrayList();

            while (s.Length > 0)
            {
                int indexOfWidthTag = s.IndexOf("width:");

                if (indexOfWidthTag > -1)
                {
                    string s2 = s.Substring(indexOfWidthTag + 6, s.Length - indexOfWidthTag - 6);

                    int indexOfEndPoint = s2.IndexOf(";"); //Look for ;
                    string widthValue;

                    if (indexOfEndPoint > -1)
                        widthValue = s2.Substring(0, indexOfEndPoint);
                    else
                    {
                        indexOfEndPoint = s2.IndexOf("'"); //Look for '

                        if (indexOfEndPoint > -1)
                            widthValue = s2.Substring(0, indexOfEndPoint);
                        else
                        {
                            indexOfEndPoint = s2.IndexOf("\""); //Look for \"

                            if (indexOfEndPoint > -1)
                                widthValue = s2.Substring(0, indexOfEndPoint);
                            else
                                widthValue = s2;
                        }
                    }

                    widthValue = widthValue.Replace("px", ""); //Remove "px"

                    int iWidthValue;
                    int.TryParse(widthValue, out iWidthValue);

                    if(iWidthValue > -1)
                        widths.Add(iWidthValue);

                    if (indexOfEndPoint > -1)
                        s = s2.Substring(indexOfEndPoint + 1, s2.Length - indexOfEndPoint - 1); //Extract the remaining of string and continue to analyze, back to begin point of loop
                    else
                        s = "";
                }
                else
                    s = ""; //Not find any "width" tag any more, finish analyzing string
            }

            return widths;
        }

        private string ExpandCollapseScript()
        {
            StringBuilder clientScript = new StringBuilder();
            string expandIcon = (ExpandIconUrl == "") ? Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images.icon_expand.gif") : ExpandIconUrl;
            string collapseIcon = (CollapseIconUrl =="")?Page.ClientScript.GetWebResourceUrl(this.GetType(), "TruongPham.WebControls.Images.icon_collapse.gif"): CollapseIconUrl;

            clientScript.AppendLine("if(document.getElementById('imgExpandCollapse_" + _controlGUID + "').src.indexOf('" + expandIcon + "') > 0) ");
            clientScript.AppendLine("{");
            clientScript.AppendLine("   getElementById(\'" + ContentRegionID + "\').style.display = 'block';");
            clientScript.AppendLine("   document.getElementById('imgExpandCollapse_" + _controlGUID + "').alt = '" + _collapseTooltip + "';");
            clientScript.AppendLine("   document.getElementById('imgExpandCollapse_" + _controlGUID + "').src = '" + collapseIcon + "';");
            clientScript.AppendLine("}");
            clientScript.AppendLine("else");
            clientScript.AppendLine("{");
            clientScript.AppendLine("   getElementById(\'" + ContentRegionID + "\').style.display = 'none';");
            clientScript.AppendLine("   document.getElementById('imgExpandCollapse_" + _controlGUID + "').alt = '" + _expandTooltip + "';");
            clientScript.AppendLine("   document.getElementById('imgExpandCollapse_" + _controlGUID + "').src = '" + expandIcon + "';");
            clientScript.AppendLine("}");
            

            return clientScript.ToString();
        }

        #endregion
    }
}
